@@ -1,6 +1,6 @@
 #!/bin/bash 
 
-set -eu
+set -e
 
 # Execute this from the top-level directory of the project (the one
 # that contains the .app bundle).  Supply the name of the .app bundle
@@ -13,6 +13,8 @@ if [ -z "$dir" ] || [ ! -d "$dir" ]; then
 	exit 2
 fi
 
+set -u
+
 entitlements=deploy/osx/Entitlements.plist
 helper_entitlements=deploy/osx/HelperEntitlements.plist
 
