@@ -43,6 +43,9 @@
 #include <QPainter>
 #include <QFileOpenEvent>
 #include <QCommandLineParser>
+#include <QSslSocket>
+#include <QFont>
+#include <QFontInfo>
 
 #include <iostream>
 #include <signal.h>
@@ -321,6 +324,30 @@ main(int argc, char **argv)
 
     QSettings settings;
 
+    QString language = QLocale::system().name();
+    SVDEBUG << "System language is: " << language << endl;
+
+    settings.beginGroup("Preferences");
+    QString prefLanguage = settings.value("locale", language).toString();
+    if (prefLanguage != QString()) language = prefLanguage;
+    settings.endGroup();
+
+    settings.beginGroup("Preferences");
+    if (!(settings.value("always-use-default-font", false).toBool())) {
+#ifdef Q_OS_WIN32
+        if (!language.startsWith("ru_")) { // + any future non-Latin i18ns
+            QFont font(QApplication::font());
+            QString preferredFamily = "Segoe UI";
+            font.setFamily(preferredFamily);
+            if (QFontInfo(font).family() == preferredFamily) {
+                font.setPointSize(9);
+                QApplication::setFont(font);
+            }
+        }
+#endif
+    }
+    settings.endGroup();
+
     settings.beginGroup("Preferences");
     // Default to using Piper server; can change in preferences
     if (!settings.contains("run-vamp-plugins-in-process")) {
@@ -329,20 +356,39 @@ main(int argc, char **argv)
     settings.endGroup();
 
     settings.beginGroup("Preferences");
-    if (showSplash && settings.value("show-splash", true).toBool()) {
+    if (showSplash) {
+        if (!settings.value("show-splash", true).toBool()) {
+            showSplash = false;
+        }
+    }
+    settings.endGroup();
+
+    if (showSplash) {
         splash = new SVSplash();
         splash->show();
         QTimer::singleShot(5000, splash, SLOT(hide()));
         application.processEvents();
     }
-    settings.endGroup();
 
     settings.beginGroup("RDF");
-    if (!settings.contains("rdf-indices")) {
-        QStringList list;
-        list << "http://www.vamp-plugins.org/rdf/plugins/index.txt";
-        settings.setValue("rdf-indices", list);
+    QStringList list;
+    bool absent = !(settings.contains("rdf-indices"));
+    QString plugIndex("http://www.vamp-plugins.org/rdf/plugins/index.txt");
+    QString packsIndex("http://www.vamp-plugins.org/rdf/packs/index.txt");
+    if (absent) {
+        list << plugIndex;
+        list << packsIndex;
+    } else {
+        list = settings.value("rdf-indices").toStringList();
+        if (!settings.contains("rdf-indices-refreshed-for-4.1")) {
+            // Packs introduced
+            if (!list.contains(packsIndex)) {
+                list << packsIndex;
+            }
+            settings.setValue("rdf-indices-refreshed-for-4.1", true);
+        }
     }
+    settings.setValue("rdf-indices", list);
     settings.endGroup();
 
     PluginPathSetter::initialiseEnvironmentVariables();
@@ -354,13 +400,9 @@ main(int argc, char **argv)
     }
     QApplication::setWindowIcon(icon);
 
-    QString language = QLocale::system().name();
-    SVDEBUG << "System language is: " << language << endl;
-
-    settings.beginGroup("Preferences");
-    QString prefLanguage = settings.value("locale", language).toString();
-    if (prefLanguage != QString()) language = prefLanguage;
-    settings.endGroup();
+    if (showSplash) {
+        application.processEvents();
+    }
 
     QTranslator qtTranslator;
     QString qtTrName = QString("qt_%1").arg(language);
@@ -389,9 +431,15 @@ main(int argc, char **argv)
 
     StoreStartupLocale();
 
-    // Make known-plugins query as early as possible after showing
-    // splash screen.
-    PluginScan::getInstance()->scan();
+#if (QT_VERSION >= 0x050400)
+    SVDEBUG << "Note: SSL library build version is: "
+            << QSslSocket::sslLibraryBuildVersionString()
+            << endl;
+#endif
+
+    if (showSplash) {
+        application.processEvents();
+    }
     
     // Permit these types to be used as args in queued signal calls
     qRegisterMetaType<PropertyContainer::PropertyName>("PropertyContainer::PropertyName");
@@ -494,6 +542,8 @@ main(int argc, char **argv)
                 << "\"" << endl;
         gui->cueOSCScript(scriptFile);
     }
+
+    SVDEBUG << "Entering main event loop" << endl;
     
     int rv = application.exec();
 
