@@ -41,7 +41,7 @@
 
 using std::vector;
 
-#define DEBUG_DOCUMENT 1
+//#define DEBUG_DOCUMENT 1
 
 //!!! still need to handle command history, documentRestored/documentModified
 
@@ -1181,7 +1181,8 @@ Document::alignModel(ModelId modelId, bool forceRecalculate)
 
     SVDEBUG << "Document::alignModel: aligning..." << endl;
     if (!rm->getAlignmentReference().isNone()) {
-        SVDEBUG << "(Note: model " << rm << " is currently aligned to model "
+        SVDEBUG << "(Note: model " << modelId
+                << " is currently aligned to model "
                 << rm->getAlignmentReference() << "; this will replace that)"
                 << endl;
     }
