@@ -50,6 +50,7 @@ class Layer;
 class WaveformLayer;
 class WaveFileModel;
 class AudioCallbackPlaySource;
+class TimeStretchWrapper;
 class AudioCallbackRecordTarget;
 class CommandHistory;
 class QMenu;
@@ -70,11 +71,11 @@ class ModelDataTableDialog;
 class QSignalMapper;
 class QShortcut;
 class AlignmentModel;
+class LayerGeometryProvider;
 
 namespace breakfastquay {
     class SystemPlaybackTarget;
     class SystemAudioIO;
-    class ResamplerWrapper;
 }
 
 /**
@@ -165,8 +166,28 @@ public:
     virtual bool saveSessionFile(QString path);
     virtual bool saveSessionTemplate(QString path);
 
-    virtual bool exportLayerTo(Layer *layer, QString path, QString &error);
+    virtual bool exportLayerToSVL(Layer *layer,
+                                  QString toPath, QString &error);
 
+    virtual bool exportLayerToMIDI(Layer *layer,
+                                   MultiSelection *selectionsToWrite, // or null
+                                   QString toPath, QString &error);
+
+    virtual bool exportLayerToRDF(Layer *layer,
+                                  QString toPath, QString &error);
+
+    virtual bool exportLayerToCSV(Layer *layer, LayerGeometryProvider *provider,
+                                  MultiSelection *selectionsToWrite, // or null
+                                  QString delimiter,
+                                  DataExportOptions options,
+                                  QString toPath, QString &error);
+
+    // Delegate to one of the above depending on extension of path,
+    // using the default export options
+    virtual bool exportLayerTo(Layer *layer, LayerGeometryProvider *provider,
+                               MultiSelection *selectionsToWrite, // or null
+                               QString toPath, QString &error);
+    
     void cueOSCScript(QString filename);
     
     /// Implementation of FrameTimer interface method
@@ -286,7 +307,6 @@ protected slots:
 
     virtual void sampleRateMismatch(sv_samplerate_t, sv_samplerate_t, bool) = 0;
     virtual void audioOverloadPluginDisabled() = 0;
-    virtual void audioTimeStretchMultiChannelDisabled() = 0;
 
     virtual void playbackFrameChanged(sv_frame_t);
     virtual void globalCentreFrameChanged(sv_frame_t);
@@ -341,9 +361,11 @@ protected slots:
     virtual void modelRegenerationWarning(QString, QString, QString) = 0;
 
     virtual void alignmentComplete(ModelId);
-    virtual void alignmentFailed(QString) = 0;
+    virtual void alignmentFailed(ModelId, QString) = 0;
 
-    virtual void rightButtonMenuRequested(Pane *, QPoint point) = 0;
+    virtual void paneRightButtonMenuRequested(Pane *, QPoint point) = 0;
+    virtual void panePropertiesRightButtonMenuRequested(Pane *, QPoint point) = 0;
+    virtual void layerPropertiesRightButtonMenuRequested(Pane *, Layer *, QPoint point) = 0;
 
     virtual void paneAdded(Pane *) = 0;
     virtual void paneHidden(Pane *) = 0;
@@ -392,7 +414,6 @@ protected:
 
     AudioCallbackPlaySource *m_playSource;
     AudioCallbackRecordTarget *m_recordTarget;
-    breakfastquay::ResamplerWrapper *m_resamplerWrapper;
     breakfastquay::SystemPlaybackTarget *m_playTarget; // only one of this...
     breakfastquay::SystemAudioIO *m_audioIO;           // ... and this exists
 
@@ -428,7 +449,7 @@ protected:
 
     bool                     m_documentModified;
     bool                     m_openingAudioFile;
-    bool                     m_abandoning;
+    bool                     m_handlingOSC;
 
     Labeller                *m_labeller;
 
