@@ -347,17 +347,21 @@ public:
         m_events.toXml(out, indent, QString("dimensions=\"2\""));
     }
 
-    QString toDelimitedDataString(QString delimiter,
-                                  DataExportOptions options,
-                                  sv_frame_t startFrame,
-                                  sv_frame_t duration) const override {
-        return m_events.toDelimitedDataString(delimiter,
-                                              options,
-                                              startFrame,
-                                              duration,
-                                              m_sampleRate,
-                                              m_resolution,
-                                              Event().withValue(0.f));
+    QVector<QString>
+    getStringExportHeaders(DataExportOptions options) const override {
+        return m_events.getStringExportHeaders(options, {});
+    }
+    
+    QVector<QVector<QString>>
+    toStringExportRows(DataExportOptions options,
+                       sv_frame_t startFrame,
+                       sv_frame_t duration) const override {
+        return m_events.toStringExportRows(options,
+                                           startFrame,
+                                           duration,
+                                           m_sampleRate,
+                                           m_resolution,
+                                           Event().withValue(0.f));
     }
   
 protected:
