@@ -102,7 +102,9 @@ void
 TransformFactory::UninstalledTransformsPopulateThread::run()
 {
     m_factory->m_populatingSlowly = true;
-    sleep(1);
+    while (!m_factory->havePopulated()) {
+        sleep(1);
+    }
     m_factory->populateUninstalledTransforms();
 }
 
@@ -324,6 +326,13 @@ TransformFactory::getTransformTypeName(TransformDescription::Type type) const
     return tr("Other");
 }
 
+bool
+TransformFactory::havePopulated()
+{
+    MutexLocker locker(&m_transformsMutex, "TransformFactory::havePopulated");
+    return m_transformsPopulated;
+}
+
 void
 TransformFactory::populateTransforms()
 {
@@ -395,6 +404,12 @@ TransformFactory::populateTransforms()
     }            
 
     m_transformsPopulated = true;
+
+#ifdef DEBUG_TRANSFORM_FACTORY
+    SVCERR << "populateTransforms exiting" << endl;
+#endif
+
+    emit transformsPopulated();
 }
 
 void
@@ -513,38 +528,38 @@ TransformFactory::populateRealTimePlugins(TransformDescriptionMap &transforms)
             continue;
         }
 
-        const RealTimePluginDescriptor *descriptor =
+        RealTimePluginDescriptor descriptor =
             factory->getPluginDescriptor(pluginId);
 
-        if (!descriptor) {
+        if (descriptor.name == "") {
             cerr << "WARNING: TransformFactory::populateTransforms: Failed to query plugin " << pluginId << endl;
             continue;
         }
         
-//!!!        if (descriptor->controlOutputPortCount == 0 ||
-//            descriptor->audioInputPortCount == 0) continue;
+//!!!        if (descriptor.controlOutputPortCount == 0 ||
+//            descriptor.audioInputPortCount == 0) continue;
 
-//        cout << "TransformFactory::populateRealTimePlugins: plugin " << pluginId << " has " << descriptor->controlOutputPortCount << " control output ports, " << descriptor->audioOutputPortCount << " audio outputs, " << descriptor->audioInputPortCount << " audio inputs" << endl;
+//        cout << "TransformFactory::populateRealTimePlugins: plugin " << pluginId << " has " << descriptor.controlOutputPortCount << " control output ports, " << descriptor.audioOutputPortCount << " audio outputs, " << descriptor.audioInputPortCount << " audio inputs" << endl;
         
-        QString pluginName = descriptor->name.c_str();
+        QString pluginName = descriptor.name.c_str();
         QString category = factory->getPluginCategory(pluginId);
-        bool configurable = (descriptor->parameterCount > 0);
-        QString maker = descriptor->maker.c_str();
+        bool configurable = (descriptor.parameterCount > 0);
+        QString maker = descriptor.maker.c_str();
         if (maker == "") maker = tr("<unknown maker>");
 
-        if (descriptor->audioInputPortCount > 0) {
+        if (descriptor.audioInputPortCount > 0) {
 
-            for (int j = 0; j < (int)descriptor->controlOutputPortCount; ++j) {
+            for (int j = 0; j < (int)descriptor.controlOutputPortCount; ++j) {
 
                 QString transformId = QString("%1:%2").arg(pluginId).arg(j);
                 QString userName;
                 QString units;
                 QString portName;
 
-                if (j < (int)descriptor->controlOutputPortNames.size() &&
-                    descriptor->controlOutputPortNames[j] != "") {
+                if (j < (int)descriptor.controlOutputPortNames.size() &&
+                    descriptor.controlOutputPortNames[j] != "") {
 
-                    portName = descriptor->controlOutputPortNames[j].c_str();
+                    portName = descriptor.controlOutputPortNames[j].c_str();
 
                     userName = tr("%1: %2")
                         .arg(pluginName)
@@ -554,7 +569,7 @@ TransformFactory::populateRealTimePlugins(TransformDescriptionMap &transforms)
                         units = unitRE.cap(1);
                     }
 
-                } else if (descriptor->controlOutputPortCount > 1) {
+                } else if (descriptor.controlOutputPortCount > 1) {
 
                     userName = tr("%1: Output %2")
                         .arg(pluginName)
@@ -593,9 +608,9 @@ TransformFactory::populateRealTimePlugins(TransformDescriptionMap &transforms)
             }
         }
 
-        if (!descriptor->isSynth || descriptor->audioInputPortCount > 0) {
+        if (!descriptor.isSynth || descriptor.audioInputPortCount > 0) {
 
-            if (descriptor->audioOutputPortCount > 0) {
+            if (descriptor.audioOutputPortCount > 0) {
 
                 QString transformId = QString("%1:A").arg(pluginId);
                 TransformDescription::Type type = TransformDescription::Effects;
@@ -604,7 +619,7 @@ TransformFactory::populateRealTimePlugins(TransformDescriptionMap &transforms)
                     .arg(pluginName)
                     .arg(maker);
 
-                if (descriptor->audioInputPortCount == 0) {
+                if (descriptor.audioInputPortCount == 0) {
                     type = TransformDescription::Generator;
                     QString description = tr("Generate audio signal using \"%1\" plugin (from %2)")
                         .arg(pluginName)
@@ -642,6 +657,8 @@ TransformFactory::populateUninstalledTransforms()
     PluginRDFIndexer::getInstance()->indexConfiguredURLs();
     if (m_exiting) return;
 
+    PluginRDFIndexer::getInstance()->performConsistencyChecks();
+    
     //!!! This will be amazingly slow
 
     QStringList ids = PluginRDFIndexer::getInstance()->getIndexedPluginIds();
@@ -653,7 +670,7 @@ TransformFactory::populateUninstalledTransforms()
         QString name = desc.getPluginName();
 #ifdef DEBUG_TRANSFORM_FACTORY
         if (name == "") {
-            cerr << "TransformFactory::populateUninstalledTransforms: "
+            SVCERR << "TransformFactory::populateUninstalledTransforms: "
                  << "No name available for plugin " << *i
                  << ", skipping" << endl;
             continue;
@@ -662,7 +679,7 @@ TransformFactory::populateUninstalledTransforms()
 
         QString description = desc.getPluginDescription();
         QString maker = desc.getPluginMaker();
-        QString infoUrl = desc.getPluginInfoURL();
+        Provider provider = desc.getPluginProvider();
 
         QStringList oids = desc.getOutputIds();
 
@@ -672,20 +689,20 @@ TransformFactory::populateUninstalledTransforms()
             
             if (m_transforms.find(tid) != m_transforms.end()) {
 #ifdef DEBUG_TRANSFORM_FACTORY
-                cerr << "TransformFactory::populateUninstalledTransforms: "
-                          << tid << " is installed; adding info url if appropriate, skipping rest" << endl;
+                SVCERR << "TransformFactory::populateUninstalledTransforms: "
+                       << tid << " is installed; adding provider if present, skipping rest" << endl;
 #endif
-                if (infoUrl != "") {
-                    if (m_transforms[tid].infoUrl == "") {
-                        m_transforms[tid].infoUrl = infoUrl;
+                if (provider != Provider()) {
+                    if (m_transforms[tid].provider == Provider()) {
+                        m_transforms[tid].provider = provider;
                     }
                 }
                 continue;
             }
 
 #ifdef DEBUG_TRANSFORM_FACTORY
-            cerr << "TransformFactory::populateUninstalledTransforms: "
-                      << "adding " << tid << endl;
+            SVCERR << "TransformFactory::populateUninstalledTransforms: "
+                   << "adding " << tid << endl;
 #endif
 
             QString oname = desc.getOutputName(*j);
@@ -726,7 +743,7 @@ TransformFactory::populateUninstalledTransforms()
             td.description = description;
             td.longDescription = longDescription;
             td.maker = maker;
-            td.infoUrl = infoUrl;
+            td.provider = provider;
             td.units = "";
             td.configurable = false;
 
@@ -739,8 +756,10 @@ TransformFactory::populateUninstalledTransforms()
     m_uninstalledTransformsPopulated = true;
 
 #ifdef DEBUG_TRANSFORM_FACTORY
-    cerr << "populateUninstalledTransforms exiting" << endl;
+    SVCERR << "populateUninstalledTransforms exiting" << endl;
 #endif
+
+    emit uninstalledTransformsPopulated();
 }
 
 Transform
@@ -753,25 +772,24 @@ TransformFactory::getDefaultTransformFor(TransformId id, sv_samplerate_t rate)
     SVDEBUG << "TransformFactory::getDefaultTransformFor: identifier \""
             << id << "\"" << endl;
     
-    Vamp::PluginBase *plugin = instantiateDefaultPluginFor(id, rate);
+    std::shared_ptr<Vamp::PluginBase> plugin = instantiateDefaultPluginFor(id, rate);
 
     if (plugin) {
         t.setPluginVersion(QString("%1").arg(plugin->getPluginVersion()));
         setParametersFromPlugin(t, plugin);
         makeContextConsistentWithPlugin(t, plugin);
-        delete plugin;
     }
 
     return t;
 }
 
-Vamp::PluginBase *
+std::shared_ptr<Vamp::PluginBase> 
 TransformFactory::instantiatePluginFor(const Transform &transform)
 {
     SVDEBUG << "TransformFactory::instantiatePluginFor: identifier \""
             << transform.getIdentifier() << "\"" << endl;
     
-    Vamp::PluginBase *plugin = instantiateDefaultPluginFor
+    std::shared_ptr<Vamp::PluginBase> plugin = instantiateDefaultPluginFor
         (transform.getIdentifier(), transform.getSampleRate());
 
     if (plugin) {
@@ -781,16 +799,18 @@ TransformFactory::instantiatePluginFor(const Transform &transform)
     return plugin;
 }
 
-Vamp::PluginBase *
+std::shared_ptr<Vamp::PluginBase> 
 TransformFactory::instantiateDefaultPluginFor(TransformId identifier,
                                               sv_samplerate_t rate)
 {
+    populateTransforms();
+
     Transform t;
     t.setIdentifier(identifier);
     if (rate == 0) rate = 44100.0;
     QString pluginId = t.getPluginIdentifier();
 
-    Vamp::PluginBase *plugin = nullptr;
+    std::shared_ptr<Vamp::PluginBase> plugin = nullptr;
 
     if (t.getType() == Transform::FeatureExtraction) {
 
@@ -824,24 +844,6 @@ TransformFactory::instantiateDefaultPluginFor(TransformId identifier,
     return plugin;
 }
 
-Vamp::Plugin *
-TransformFactory::downcastVampPlugin(Vamp::PluginBase *plugin)
-{
-    Vamp::Plugin *vp = dynamic_cast<Vamp::Plugin *>(plugin);
-    if (!vp) {
-//        cerr << "makeConsistentWithPlugin: not a Vamp::Plugin" << endl;
-        vp = dynamic_cast<Vamp::PluginHostAdapter *>(plugin); //!!! why?
-}
-    if (!vp) {
-//        cerr << "makeConsistentWithPlugin: not a Vamp::PluginHostAdapter" << endl;
-        vp = dynamic_cast<Vamp::HostExt::PluginWrapper *>(plugin); //!!! no, I mean really why?
-    }
-    if (!vp) {
-//        cerr << "makeConsistentWithPlugin: not a Vamp::HostExt::PluginWrapper" << endl;
-    }
-    return vp;
-}
-
 bool
 TransformFactory::haveTransform(TransformId identifier)
 {
@@ -873,12 +875,12 @@ TransformFactory::getTransformUnits(TransformId identifier)
     } else return "";
 }
 
-QString
-TransformFactory::getTransformInfoUrl(TransformId identifier)
+Provider
+TransformFactory::getTransformProvider(TransformId identifier)
 {
     if (m_transforms.find(identifier) != m_transforms.end()) {
-        return m_transforms[identifier].infoUrl;
-    } else return "";
+        return m_transforms[identifier].provider;
+    } else return {};
 }
 
 Vamp::Plugin::InputDomain
@@ -894,12 +896,12 @@ TransformFactory::getTransformInputDomain(TransformId identifier)
         return Vamp::Plugin::TimeDomain;
     }
 
-    Vamp::Plugin *plugin =
-        downcastVampPlugin(instantiateDefaultPluginFor(identifier, 0));
+    std::shared_ptr<Vamp::Plugin> plugin =
+        std::dynamic_pointer_cast<Vamp::Plugin>
+        (instantiateDefaultPluginFor(identifier, 0));
 
     if (plugin) {
         Vamp::Plugin::InputDomain d = plugin->getInputDomain();
-        delete plugin;
         return d;
     }
 
@@ -922,13 +924,15 @@ TransformFactory::getTransformChannelRange(TransformId identifier,
 
     if (RealTimePluginFactory::instanceFor(id)) {
 
-        const RealTimePluginDescriptor *descriptor = 
+        RealTimePluginDescriptor descriptor = 
             RealTimePluginFactory::instanceFor(id)->
             getPluginDescriptor(id);
-        if (!descriptor) return false;
+        if (descriptor.name == "") {
+            return false;
+        }
 
-        min = descriptor->audioInputPortCount;
-        max = descriptor->audioInputPortCount;
+        min = descriptor.audioInputPortCount;
+        max = descriptor.audioInputPortCount;
 
         return true;
 
@@ -949,7 +953,7 @@ TransformFactory::getTransformChannelRange(TransformId identifier,
 
 void
 TransformFactory::setParametersFromPlugin(Transform &transform,
-                                          Vamp::PluginBase *plugin)
+                                          std::shared_ptr<Vamp::PluginBase> plugin)
 {
     Transform::ParameterMap pmap;
 
@@ -976,8 +980,8 @@ TransformFactory::setParametersFromPlugin(Transform &transform,
         transform.setProgram(plugin->getCurrentProgram().c_str());
     }
 
-    RealTimePluginInstance *rtpi =
-        dynamic_cast<RealTimePluginInstance *>(plugin);
+    std::shared_ptr<RealTimePluginInstance> rtpi =
+        std::dynamic_pointer_cast<RealTimePluginInstance>(plugin);
 
     Transform::ConfigurationMap cmap;
 
@@ -997,14 +1001,14 @@ TransformFactory::setParametersFromPlugin(Transform &transform,
 
 void
 TransformFactory::setPluginParameters(const Transform &transform,
-                                      Vamp::PluginBase *plugin)
+                                      std::shared_ptr<Vamp::PluginBase> plugin)
 {
     //!!! check plugin & API version (see e.g. PluginXml::setParameters)
 
     //!!! check that this is the right plugin!
 
-    RealTimePluginInstance *rtpi =
-        dynamic_cast<RealTimePluginInstance *>(plugin);
+    std::shared_ptr<RealTimePluginInstance> rtpi =
+        std::dynamic_pointer_cast<RealTimePluginInstance>(plugin);
 
     if (rtpi) {
         const Transform::ConfigurationMap &cmap = transform.getConfiguration();
@@ -1035,9 +1039,10 @@ TransformFactory::setPluginParameters(const Transform &transform,
 
 void
 TransformFactory::makeContextConsistentWithPlugin(Transform &transform,
-                                                  Vamp::PluginBase *plugin)
+                                                  std::shared_ptr<Vamp::PluginBase> plugin)
 {
-    const Vamp::Plugin *vp = downcastVampPlugin(plugin);
+    std::shared_ptr<Vamp::Plugin> vp =
+        std::dynamic_pointer_cast<Vamp::Plugin>(plugin);
 
     if (!vp) {
         // time domain input for real-time effects plugin
@@ -1078,8 +1083,7 @@ TransformFactory::getPluginConfigurationXml(const Transform &t)
     SVDEBUG << "TransformFactory::getPluginConfigurationXml: identifier \""
             << t.getIdentifier() << "\"" << endl;
 
-    Vamp::PluginBase *plugin = instantiateDefaultPluginFor
-        (t.getIdentifier(), 0);
+    auto plugin = instantiateDefaultPluginFor(t.getIdentifier(), 0);
     if (!plugin) {
         SVDEBUG << "TransformFactory::getPluginConfigurationXml: "
                 << "Unable to instantiate plugin for transform \""
@@ -1091,7 +1095,6 @@ TransformFactory::getPluginConfigurationXml(const Transform &t)
 
     QTextStream out(&xml);
     PluginXml(plugin).toXml(out);
-    delete plugin;
 
     return xml;
 }
@@ -1103,8 +1106,7 @@ TransformFactory::setParametersFromPluginConfigurationXml(Transform &t,
     SVDEBUG << "TransformFactory::setParametersFromPluginConfigurationXml: identifier \""
             << t.getIdentifier() << "\"" << endl;
 
-    Vamp::PluginBase *plugin = instantiateDefaultPluginFor
-        (t.getIdentifier(), 0);
+    auto plugin = instantiateDefaultPluginFor(t.getIdentifier(), 0);
     if (!plugin) {
         SVDEBUG << "TransformFactory::setParametersFromPluginConfigurationXml: "
                 << "Unable to instantiate plugin for transform \""
@@ -1114,7 +1116,6 @@ TransformFactory::setParametersFromPluginConfigurationXml(Transform &t,
 
     PluginXml(plugin).setParametersFromXml(xml);
     setParametersFromPlugin(t, plugin);
-    delete plugin;
 }
 
 TransformFactory::SearchResults
@@ -1130,11 +1131,36 @@ TransformFactory::search(QStringList keywords)
 {
     populateTransforms();
 
+    SearchResults results = searchUnadjusted(keywords);
+    
     if (keywords.size() > 1) {
-        // Additional score for all keywords in a row
-        keywords.push_back(keywords.join(" "));
+
+        // If there are any hits for all keywords in a row, put them
+        // in (replacing previous hits for the same transforms) but
+        // ensure they score more than any of the others
+
+        int maxScore = 0;
+        for (auto r: results) {
+            if (r.second.score > maxScore) {
+                maxScore = r.second.score;
+            }
+        }
+
+        QStringList oneBigKeyword;
+        oneBigKeyword << keywords.join(" ");
+        SearchResults oneBigKeywordResults = searchUnadjusted(oneBigKeyword);
+        for (auto r: oneBigKeywordResults) {
+            results[r.first] = r.second;
+            results[r.first].score += maxScore;
+        }
     }
 
+    return results;
+}
+
+TransformFactory::SearchResults
+TransformFactory::searchUnadjusted(QStringList keywords)
+{
     SearchResults results;
     TextMatcher matcher;
 
@@ -1197,6 +1223,26 @@ TransformFactory::search(QStringList keywords)
         if (match.score > 0) results[i->first] = match;
     }
 
+#ifdef DEBUG_TRANSFORM_FACTORY
+    SVCERR << "TransformFactory::search: keywords are: " << keywords.join(", ")
+           << endl;
+    int n = int(results.size()), i = 1;
+    SVCERR << "TransformFactory::search: results (" << n << "):" << endl;
+    
+    for (const auto &r: results) {
+        QStringList frags;
+        for (const auto &f: r.second.fragments) {
+            frags << QString("{\"%1\": \"%2\"}").arg(f.first).arg(f.second);
+        }
+        SVCERR << "[" << i << "/" << n << "] id " << r.first
+               << ": score " << r.second.score
+               << ", key " << r.second.key << ", fragments "
+               << frags.join(";") << endl;
+        ++i;
+    }
+    SVCERR << endl;
+#endif
+    
     return results;
 }
 
