@@ -862,7 +862,7 @@ View::addLayer(Layer *layer)
     pbr.cancel = cancel;
     pbr.bar = pb;
     pbr.lastStallCheckValue = 0;
-    pbr.stallCheckTimer = new QTimer();
+    pbr.stallCheckTimer = new QTimer(this);
     connect(pbr.stallCheckTimer, SIGNAL(timeout()), this,
             SLOT(progressCheckStalledTimerElapsed()));
 
@@ -2199,8 +2199,19 @@ View::setPaintFont(QPainter &paint)
     }
 
     QFont font(paint.font());
-    font.setPointSize(Preferences::getInstance()->getViewFontSize()
-                      * scaleFactor);
+    int pointSize = Preferences::getInstance()->getViewFontSize() * scaleFactor;
+    font.setPointSize(pointSize);
+
+    int h = height();
+    int fh = QFontMetrics(font).height();
+    if (pointSize > 6) {
+        if (h < fh * 2.1) {
+            font.setPointSize(pointSize - 2);
+        } else if (h < fh * 3.1) {
+            font.setPointSize(pointSize - 1);
+        }
+    }
+    
     paint.setFont(font);
 }
 
@@ -2992,21 +3003,24 @@ View::drawMeasurementRect(QPainter &paint, const Layer *topLayer, QRect r,
 }
 
 bool
-View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
+View::waitForLayersToBeReady()
 {
-    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
-    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
-
-    int w = x1 - x0;
-
-    sv_frame_t origCentreFrame = m_centreFrame;
-
     bool someLayersIncomplete = false;
 
+#ifdef DEBUG_VIEW
+    SVDEBUG << "View::waitForLayersToBeReady: checking completion" << endl;
+#endif
+    
     for (LayerList::iterator i = m_layerStack.begin();
          i != m_layerStack.end(); ++i) {
 
         int c = (*i)->getCompletion(this);
+
+#ifdef DEBUG_VIEW
+        SVDEBUG << "layer " << (*i)->getLayerPresentationName() << " says "
+                << c << endl;
+#endif
+        
         if (c < 100) {
             someLayersIncomplete = true;
             break;
@@ -3022,6 +3036,10 @@ View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
 
         while (layerCompletion < 100) {
 
+#ifdef DEBUG_VIEW
+            SVDEBUG << "View::render: checking completion (again)" << endl;
+#endif
+    
             for (LayerList::iterator i = m_layerStack.begin();
                  i != m_layerStack.end(); ++i) {
 
@@ -3029,6 +3047,11 @@ View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
                 if (i == m_layerStack.begin() || c < layerCompletion) {
                     layerCompletion = c;
                 }
+
+#ifdef DEBUG_VIEW
+                SVDEBUG << "layer " << (*i)->getLayerPresentationName() << " says "
+                        << c << ", layerCompletion now " << layerCompletion << endl;
+#endif
             }
 
             if (layerCompletion >= 100) break;
@@ -3044,6 +3067,33 @@ View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
         }
     }
 
+#ifdef DEBUG_VIEW
+    SVDEBUG << "View::waitForLayersToBeReady: ok, we're ready" << endl;
+#endif
+
+    return true;
+}
+
+bool
+View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
+{
+    int x0 = int(round(m_zoomLevel.framesToPixels(double(f0))));
+    int x1 = int(round(m_zoomLevel.framesToPixels(double(f1))));
+
+    int w = x1 - x0;
+
+#ifdef DEBUG_VIEW
+    SVDEBUG << "View::render: Render request is for frames " << f0
+            << " to " << f1 << " (pixels " << x0 << " to " << x1
+            << ", width " << w << ")" << endl;
+#endif
+
+    if (!waitForLayersToBeReady()) {
+        return false;
+    }
+
+    sv_frame_t origCentreFrame = m_centreFrame;
+    
     QProgressDialog progress(tr("Rendering image..."),
                              tr("Cancel"), 0, w / width(), this);
 
@@ -3100,6 +3150,10 @@ View::render(QPainter &paint, int xorigin, sv_frame_t f0, sv_frame_t f1)
 QImage *
 View::renderToNewImage()
 {
+    if (!waitForLayersToBeReady()) {
+        return nullptr;
+    }
+    
     sv_frame_t f0 = getModelsStartFrame();
     sv_frame_t f1 = getModelsEndFrame();
 
